"""
Configuration models for Django allowing config management with auditing.
"""

from __future__ import absolute_import, unicode_literals

__version__ = '2.0.1'

default_app_config = 'config_models.apps.ConfigModelsConfig'  # pylint: disable=invalid-name
